##############################################################################
# Chapter 6                                                                  #
# Sanchez, J. Time Series for Data Scientists, CUP 2023.                     #
# Case Study                                                                 #
# JSM-cgmprogram.R in Base R version                                         #
#  Last updated: Feb 23                                                                          #
##############################################################################



#install.packages("fpp3")
#install.packages("tidyverse")
#install.packages("tsibble")
#install.packages("lubridate")
#install.packages("readr")
#install.packages("fable")
#install.packages("tsibbledata")
#install.packages("gridExtra")
#install.packages("prophet")
library(tidyverse)
library(tsibble)
library(fable)
library(lubridate)
library(readr)
library(ggplot2)
library(tsibbledata)
library(fpp3)
library(gridExtra)
library(prophet)

### traditional exploration

##  Reading glucose reading data every five minutes on May 1-June 26 2021
## Notice that we will be focusing in the weekly and daily seasonality, since 
## we only have barely two months of data. 

    data=unique(read.csv("cgmdataonly.csv", header=T))
    head(data)  # Notice the time stamp 5 minute frequency
     dim(data)
     data.ts=ts(data$cgmreadings) # frequency set as 1,for convenience. 
 
     
        ## diffdata=diff(data.ts) # if interested in change of blood glucose
 #decompose(data.ts) # This will not work with this data without setting a freq
     
## Quick draft inspection 
     
     plot.ts(data.ts, main="CGM readings May 1-June 26y,\n 2021-every five min")
     head(data.ts)
     start(data.ts)
     end(data.ts)
     frequency(data.ts)
     # diffdata=diff(data.ts, lag=1)
     # plot.ts(diffdata)   #Look if interested in diff
     
   par(mfrow=c(2,1))  
 acf(data.ts,main="ACF of CGM readings May 1-June 26,\n 2021-every five min")
  pacf(data.ts,main="PACF of CGM readings May 1-June 26,\n 2021-every five min")
      dev.off() 
      
  # par(mfrow=c(2,1))  #if interested in change
  #     acf(diffdata)
  #     pacf(diffdata)
  #   dev.off() 
     
    
     hist(data$cgmreadings, main="Histogram of CGM readings, 
          May 1-June 26,\n 2021-every five min ")
 

## Read again the data for more analysis. Make sure duplicates are removed

  cgm =unique(readr::read_csv("cgmdataonly.csv",
                            col_types=list(col_datetime(),col_double())))
   tz(cgm$date_time)="America/Los_Angeles"
   class(cgm$date_time)
   cgm
   tz(cgm$date_time)

## Extract time features
   cgm$weekday=wday(cgm$date_time, label=TRUE)
   dayofmonth=mday(cgm$date_time)
   head(dayofmonth)
   cgm$hour=hour(cgm$date_time)
   cgm$min5interval=factor(minute(cgm$date_time))
   cgm$week=week(cgm$date_time) # extracts week of the 
   head(cgm)
   tail(cgm)
   class(cgm$min5interval)
  cgm[21:30,]
   table(cgm$min5interval)

                       

## Plot time components
   
   
      # the following graph is not in the Case Study of Chapter 6
 ## interstitial blood glucose lags behind real blood glucoe, 
## and interstitial blood glucose is generally lower
   ggplot(data=cgm)+
    geom_boxplot(mapping=aes(group=weekday, x=weekday,y=cgmreadings)) +
     labs(
       title=paste("During the second half of the week the patient generally has slightly lower  blood glucose levels"), 
       caption="CGM readings May 1st-June 26, 2021", 
       x="Day of the week", 
       y="Interstitial blood glucose level readings"
     )
  
              #########  ggplot version of Figure 6.18 ############
   
   ggplot(data=cgm)+
    geom_boxplot(mapping=aes(group=hour, x=hour,y=cgmreadings)) +
    labs(
      title=paste("There is clearly an hour-of-the-day-effect in the CGM blood glucose readings"), 
caption="CGM readings May 1st-June 26, 2021", 
      x="Hour of the day", 
      y="Interstitial blood glucose level readings"
    )
  
           ############ ggplot version of Figure 6.17 ##############

  ggplot(data=cgm)+
    geom_line(mapping=aes( x=date_time,y=cgmreadings),alpha=0.3) +
    geom_smooth(mapping=aes(x=date_time, y=cgmreadings, color="white", size=0.3), show.legend=FALSE)+
    labs(
      title=paste("The two-months trend (in red) is nonlinear, but with no tendency to go up or down"), 
      caption="CGM readings May 1st-June 26, 2021", 
      x="Day and time of the day (5 minutes intervals)", 
      y="Interstitial blood glucose level readings"
    )+
    geom_hline(yintercept=c(70,160), color="green",size=2)
  
  
  ## the following plot is not in the case study in Chapter 6
   ## We would need many more weeks to get a realistic pattern for a year
  ggplot(data=cgm)+
    geom_boxplot(mapping=aes(group=week, x=week,y=cgmreadings)) +
    labs(
      title=paste("Some weeks have more variability in blood glucose readings than others"), 
      caption="CGM readings May 1st-June 26, 2021", 
      x="Week of the year", 
      y="Interstitial blood glucose level readings"
    )+
    geom_hline(yintercept=c(70,160), color="green",size=2)
  
  
                       ########## A more detailed view of  Figure 6.19 #########
  
 #### Reshape from long to wide form by week for spagheti plots. 
  
## Another view of the daily seasonality, with a third variable, weekday
  ggplot(data=cgm)+
    geom_point(mapping=aes(x=hour,y=cgmreadings,group=weekday,linetype=weekday,color=weekday)) +
    labs(
      title=paste("Some weeks have more variability in blood glucose readings than others"), 
      caption="CGM readings May 1st-June 26, 2021", 
      x="Week of the year", 
      y="Interstitial blood glucose level readings"
    )+
    geom_hline(yintercept=c(70,160), color="green",size=2)
  
  
  
  
  
  ## See how often there is collection of data. 
  ## The following shows some abnormality in the data collection or reading
  ## Can the reader see what it is? See Chapter 10's case study on cgm 
  ## for more on this. 
  
  
 #   ggplot(data=cgm)+geom_boxplot(mapping=aes(group=min5interval,
##   x=min5interval,y=cgmreadings))   
    
  #  hist(cgm$hour)
  

cgm1=cgm
cgm1
cgm2=unique(as_tsibble(cgm1,regular=F, index=date_time) ) 
cgm2


           ###### Figure 6.19 ############

#### Try to do a plot for each week, with each day in 
## a spaghetti plot. 
cgm2=unique(as_tsibble(cgm1,regular=F, index=date_time) ) 
cgm2

cgm2 %>% group_by(week) %>% 
  ggplot(aes(x = hour, y = cgmreadings, group = weekday)) +
  geom_smooth(aes(col = weekday),se=FALSE) +
  facet_wrap(~week)+
  # could use face_grid
labs(
  title=paste("Smoothed Weekly patterns of blood glucose readings vary"), 
  caption="CGM readings May 1st-June 26, 2021", 
  x="Hour", 
  y="Interstitial blood glucose level readings"
)


              ###### Figures 6.20 and 6.21 ##########

#### Fitting and Forecasting with Prophet
cgmforprophet =unique(readr::read_csv("cgmdataonly.csv",
                                      col_types=list(col_datetime(),col_double())))
cgmforprophet
cgmforprophet$ds=cgmforprophet$date_time
cgmforprophet$y=cgmforprophet$cgmreadings
cgmnew=cgmforprophet[,3:4]
cgmnew

m=prophet(cgmforprophet,daily.seasonality=TRUE)
#?prophet::make_future_dataframe
# the following produces daily forecasts
future <- make_future_dataframe(m, periods = 20)

future <- make_future_dataframe(m, periods = 168,freq=3600)
head(future)
forecast <- predict(m, future)
head(forecast)

  ##### ggplot version of Figure 6.20 ############

plot(m,forecast,xlab="Time", 
     ylab="Interstitial blood glucose level readings",
     main="Prophet fit and forecast")

  ##### Figure 6.21 #########

prophet_plot_components(m,forecast)



    ##  Extra ###### To see detail, a dyplot always helps ########



dyplot.prophet(m,forecast,main="Prophet fit and forecast",
               ylab="Interstitial blood glucose level readings")



